==> node-server-0.0.4.tgz: package/lib/Bootstrap.js <==
var ChildProcess = require("child_process");
var NetBinding = process.binding("net"); 
var Net = require("net");
var Fs = require("fs");

var ApplicationConfig = require("./ApplicationConfig.js").ApplicationConfig;
var BootConfigManager = require("./BootConfigManager.js").BootConfigManager;
var Exit = require("./Exit.js").Exit;
var Application = require("./Application.js").Application;

exports.Bootstrap = function(argv){
    var params = argv.argv;
    try{
        var configuration = new ApplicationConfig(params);
    }
    catch(err){
        //console.dir(argv);
        console.log(err);
        argv.help();
    }
    var bootConfig = new BootConfigManager();
    this.run = function(){
        if(params.start && !params.stop){
            process.chdir(params.path);
			require.paths.unshift(process.cwd());
			require.paths.unshift(process.cwd() + '/lib/');
			require.paths.unshift(process.cwd() + '/node_modules/');
            if(params.mode == 'master' && !params.debug){
                //master mode
                var fd = this.getFd();
                var socket = this.getSocket(params.location, fd);
                var that = this;
                //otwieram dev null
                var devnull = Fs.openSync('/dev/null', "w+");
                //ustawiam token
                process.env.token = process.pid;
                //spawn-uje proces potomny
                var child = ChildProcess.spawn(
                    'node', [params.script, './', "-l", params.location.host + ":" + params.location.port, "--mode", "slave"],
                    { cwd: process.cwd(),
                      env: process.env,
                      //customFds: [socket[1], devnull, devnull],
                      customFds: [socket[1], -1, -1],
                      setsid: true
                    }
                );
                child.stdout.on('data', function(data){
                   //console.log("Slave:", data.toString());
                   var data = data.toString();
                   try{
                       var jsonData = JSON.parse(data);
                       if(jsonData.hasOwnProperty("result") && jsonData.result.hasOwnProperty("pid") && jsonData.result.hasOwnProperty("instance")){
                           bootConfig.setApplicationInfo(params.path, params.location.host, params.location.port, jsonData.result.pid, jsonData.result.instance);
                           bootConfig.save();
                          
                           child.stdout.open(devnull);
                           child.stderr.open(devnull);
                           
                           Exit.success(jsonData.result.instance, jsonData.result.pid);
                       }
                       else if(jsonData.hasOwnProperty("error")){
                           Exit.applicationError(jsonData.error.data.code, jsonData.error.data.message);
                       }
                   }
                   catch(e){
                       //console.log("Master:", e.message, e.stack);
                   };
                });
                //przekazuje socket dziecku
                child.master = new Net.Stream(socket[0], 'unix');
                child.master.write("tcp", "ascii", fd);
                //czekam na informacje zwrotną parenta
                //bootConfig.onUpdate(process.cwd(), params.location.host, params.location.port, process.env.token, function(info){
                    //Exit.success(info.id, info.pid);
                //});
                //bootConfig.onUpdateTimeout(process.env.token, function(){
                    //Exit.applicationError(3001, "Starting given server class failed.");
                //});
            }
            else{
                // slave / stand alone mode
                var app = new Application(configuration);
                app.location = params.location;
                app.mode = params.mode;
                app.debug = params.debug;
                app.run();
            }   
        }
        else{
            //killing mode
            var appInfo = bootConfig.getApplicationInfo(params.path, params.location.host, params.location.port);
            var count = appInfo.length;
            //console.log(appInfo);
            for(var i = 0; i < count; i++){
                process.kill(appInfo[i].pid);
                //bootConfig.removeApplicationInfo(params.path, appInfo[i].pid);   
            }
            //bootConfig.save();
        }
    };
    this.getFd = function(){
        return NetBinding.socket("tcp4");
    };
    this.getSocket = function (location, fd){
        try{
            NetBinding.bind(fd, location.port, location.host);
            NetBinding.listen(fd, 128);
            return NetBinding.socketpair();   
        }
        catch(err){
            Exit.systemError(98, "Address already in use");
        }
    };
};

==> node-server-0.0.17.tgz: package/lib/Bootstrap.js <==
var ChildProcess = require("child_process");
var NetBinding = process.binding("net"); 
var Net = require("net");
var Fs = require("fs");

var ApplicationConfig = require("./ApplicationConfig.js").ApplicationConfig;
var BootConfigManager = require("./BootConfigManager.js").BootConfigManager;
var Exit = require("./Exit.js").Exit;
var Application = require("./Application.js").Application;

exports.Bootstrap = function(argv){
    var params = argv.argv;
    try{
        var configuration = new ApplicationConfig(params);
    }
    catch(err){
        //console.dir(argv);
        console.log(err);
        argv.help();
    }
    var bootConfig = new BootConfigManager();
    this.run = function(){
        if(params.start && !params.stop){
            process.chdir(params.path);
			require.paths.unshift(process.cwd());
			require.paths.unshift(process.cwd() + '/lib/');
			require.paths.unshift(process.cwd() + '/node_modules/');
            if(params.mode == 'master' && !params.debug){
                //master mode
                var fd = this.getFd();
                var socket = this.getSocket(params.location, fd);
                var that = this;
                //otwieram dev null
                var devnull = Fs.openSync('/dev/null', "w+");
                //ustawiam token
                process.env.token = process.pid;
                process.env['node-server'] = {};
                //for future use
                //spawn-uje proces potomny
                var child = ChildProcess.spawn(
                    'node', [params.script, './', "-l", params.location.host + ":" + params.location.port, "--mode", "slave", "--debuglog", params.debuglog],
                    { cwd: process.cwd(),
                      env: process.env,
                      //customFds: [socket[1], devnull, devnull],
                      customFds: [socket[1], -1, -1],
                      setsid: true
                    }
                );
                child.stdout.on('data', function(data){
                   //console.log("Slave:", data.toString());
                   var data = data.toString();
                   try{
                       var jsonData = JSON.parse(data);
                       if(jsonData.hasOwnProperty("result") && jsonData.result.hasOwnProperty("pid") && jsonData.result.hasOwnProperty("instance")){
                           bootConfig.setApplicationInfo(params.path, params.location.host, params.location.port, jsonData.result.pid, jsonData.result.instance);
                           bootConfig.save();
                          
                           child.stdout.open(devnull);
                           child.stderr.open(devnull);
                           
                           Exit.success(jsonData.result.instance, jsonData.result.pid);
                       }
                       else if(jsonData.hasOwnProperty("error")){
                           Exit.applicationError(jsonData.error.data.code, jsonData.error.data.message);
                       }
                   }
                   catch(e){
                       //console.log("Master:", e.message, e.stack);
                   };
                });
                //przekazuje socket dziecku
                child.master = new Net.Stream(socket[0], 'unix');
                child.master.write("tcp", "ascii", fd);
                //czekam na informacje zwrotną parenta
                //bootConfig.onUpdate(process.cwd(), params.location.host, params.location.port, process.env.token, function(info){
                    //Exit.success(info.id, info.pid);
                //});
                //bootConfig.onUpdateTimeout(process.env.token, function(){
                    //Exit.applicationError(3001, "Starting given server class failed.");
                //});
            }
            else{
                // slave / stand alone mode
                var app = new Application(configuration);
                app.location = params.location;
                app.mode = params.mode;
                app.debug = params.debug;
                app.enableDebugLog = params.debuglog;
                app.run();
            }   
        }
        else{
            //killing mode
            var appInfo = bootConfig.getApplicationInfo(params.path, params.location.host, params.location.port);
            var count = appInfo.length;
            //console.log(appInfo);
            for(var i = 0; i < count; i++){
                process.kill(appInfo[i].pid);
                //bootConfig.removeApplicationInfo(params.path, appInfo[i].pid);   
            }
            //bootConfig.save();
        }
    };
    this.getFd = function(){
        return NetBinding.socket("tcp4");
    };
    this.getSocket = function (location, fd){
        try{
            NetBinding.bind(fd, location.port, location.host);
            NetBinding.listen(fd, 128);
            return NetBinding.socketpair();   
        }
        catch(err){
            Exit.systemError(98, "Address already in use");
        }
    };
};
